pub struct PipeRef { /* private fields */ }Expand description
A reference to a given pipe port.
We need this type because the std::io::bufreader requires us to give out the ownership of the
inner object to the bufreader. However a pipe port object should be used for each servlet
activation, so we basically can not give it out. So we implement this reference type, so that we
can give the ownership of this object to bufreader without destory the orignal pipe port object.